@@ -6,8 +6,8 @@
 ### to a collection of NG groups, or, more formally, a bipartite graph
 ### between integer sets [1, NO] and [1, NG]. Objects mapped to a given group
 ### are said to belong to, or to be assigned to, or to be in that group.
-### Additionally, the objects in each group are ordered. So for example
-### the 2 following groupings are considered different:
+### Additionally, the objects in each group are ordered. So for example the
+### 2 following groupings are considered different:
 ###
 ###   Grouping 1: NG = 3, NO = 5
 ###               group   objects
@@ -33,16 +33,29 @@ setClass("Grouping", contains="IntegerList", representation("VIRTUAL"))
 
 setGeneric("nobj", function(x) standardGeneric("nobj"))
 
-setGeneric("grouplength", signature="x",
-    function(x, i=NULL) standardGeneric("grouplength")
+setGeneric("grouplengths", signature="x",
+    function(x, i=NULL) standardGeneric("grouplengths")
 )
 
-setMethod("grouplength", "Grouping",
+.subset_by_integer <- function(x, i=NULL)
+{
+    if (is.null(i))
+        return(x)
+    if (!is.numeric(i))
+        stop(wmsg("subscript must be NULL or an integer vector"))
+    if (!is.integer(i))
+        i <- as.integer(i)
+    x_len <- length(x)
+    if (S4Vectors:::anyMissingOrOutside(i, -x_len, x_len))
+        stop(wmsg("subscript contains NAs or out of bounds indices"))
+    x[i]
+}
+
+setMethod("grouplengths", "Grouping",
     function(x, i=NULL)
     {
-        if (is.null(i))
-            i <- seq_len(length(x))
-        sapply(i, function(ii) length(x[[i]]))
+        x_grouplens <- elementNROWS(x)
+        .subset_by_integer(x_grouplens, i)
     }
 )
 
@@ -57,7 +70,7 @@ setMethod("show", "Grouping",
             sep="")
         if (NG == 0L)
             return(invisible(NULL))
-        empty_groups <- which(grouplength(object) == 0L)
+        empty_groups <- which(grouplengths(object) == 0L)
         cat("Nb of empty groups: ", length(empty_groups),
             " (", 100.00 * length(empty_groups) / NG, "%)\n", sep="")
     }
@@ -69,9 +82,11 @@ setMethod("show", "Grouping",
 ### -------------------------
 
 ### A ManyToOneGrouping object represents a grouping where every object in
-### the collection is mapped to one group and only one.
+### the collection belongs to exactly one group.
 setClass("ManyToOneGrouping", contains="Grouping", representation("VIRTUAL"))
 
+setMethod("nobj", "ManyToOneGrouping", function(x) sum(grouplengths(x)))
+
 setGeneric("members", signature="x",
     function(x, i) standardGeneric("members")
 )
@@ -104,23 +119,15 @@ setGeneric("togroup", signature="x",
     function(x, j=NULL) standardGeneric("togroup")
 )
 
-### The default method works on any object 'x' for which 'elementLengths(x)'
-### works (e.g. Partitioning, List, list). Not very efficient.
-setMethod("togroup", "ANY",
+### Works on any ManyToOneGrouping object 'x' for which unlist() and
+### elementNROWS() work.
+setMethod("togroup", "ManyToOneGrouping",
     function(x, j=NULL)
     {
-        elt_len <- elementLengths(x)
-        to_group <- rep.int(seq_len(length(elt_len)), elt_len)
-        if (is.null(j))
-            return(to_group)
-        if (!is.numeric(j))
-            stop(wmsg("subscript 'j' must be a vector of integers or NULL"))
-        if (!is.integer(j))
-            j <- as.integer(j)
-        bound <- length(to_group)
-        if (S4Vectors:::anyMissingOrOutside(j, -bound, bound))
-            stop(wmsg("subscript 'j' contains NAs or out of bounds indices"))
-        to_group[j]
+        x_togroup <- unlist(x, use.names=FALSE)
+        x_eltNROWS <- elementNROWS(x)
+        x_togroup[x_togroup] <- rep.int(seq_along(x_eltNROWS), x_eltNROWS)
+        .subset_by_integer(x_togroup, j)
     }
 )
 
@@ -129,8 +136,7 @@ setGeneric("togrouplength", signature="x",
 )
 
 setMethod("togrouplength", "ManyToOneGrouping",
-    function(x, j=NULL)
-        grouplength(x, togroup(x, j))
+    function(x, j=NULL) grouplengths(x, togroup(x, j))
 )
 
 
@@ -195,21 +201,12 @@ setMethod("getListElement", "H2LGrouping",
 
 ### Should be more efficient than the default method for ManyToOneGrouping
 ### objects.
-setMethod("grouplength", "H2LGrouping",
+setMethod("grouplengths", "H2LGrouping",
     function(x, i=NULL)
     {
-        group_length <- elementLengths(x@low2high) + 1L
-        group_length[!is.na(x@high2low)] <- 0L
-        if (is.null(i))
-            return(group_length)
-        if (!is.numeric(i))
-            stop(wmsg("subscript 'i' must be a vector of integers or NULL"))
-        if (!is.integer(i))
-            i <- as.integer(i)
-        bound <- length(group_length)
-        if (S4Vectors:::anyMissingOrOutside(i, -bound, bound))
-            stop(wmsg("subscript 'i' contains NAs or out of bounds indices"))
-        group_length[i]
+        x_grouplens <- elementNROWS(x@low2high) + 1L
+        x_grouplens[!is.na(x@high2low)] <- 0L
+        .subset_by_integer(x_grouplens, i)
     }
 )
 
@@ -237,18 +234,9 @@ setMethod("vmembers", "H2LGrouping",
 setMethod("togroup", "H2LGrouping",
     function(x, j=NULL)
     {
-        to_group <- x@high2low
-        to_group[is.na(to_group)] <- which(is.na(to_group))
-        if (is.null(j))
-            return(to_group)
-        if (!is.numeric(j))
-            stop(wmsg("subscript 'j' must be a vector of integers or NULL"))
-        if (!is.integer(j))
-            j <- as.integer(j)
-        bound <- length(to_group)
-        if (S4Vectors:::anyMissingOrOutside(j, -bound, bound))
-            stop(wmsg("subscript 'j' contains NAs or out of bounds indices"))
-        to_group[j]
+        x_togroup <- x@high2low
+        x_togroup[is.na(x_togroup)] <- which(is.na(x_togroup))
+        .subset_by_integer(x_togroup, j)
     }
 )
 
@@ -292,10 +280,10 @@ setMethod("grouprank", "H2LGrouping",
 ### An important property of togrouprank() is that:
 ###   togrouprank(x, neg_idx)
 ### and
-###   seq_len(length(neg_idx))
+###   seq_along(neg_idx)
 ### are identical, where 'neg_idx' is the vector of the indices of
 ### the non-empty groups i.e.
-###   neg_idx <- which(grouplength(x) != 0L)
+###   neg_idx <- which(grouplengths(x) != 0L)
 setGeneric("togrouprank", signature="x",
      function(x, j=NULL) standardGeneric("togrouprank")
 )
@@ -379,10 +367,7 @@ setValidity("H2LGrouping",
                   "only accepts 'incomparables=FALSE'"))
     !is.na(high2low(x))
 }
-### S3/S4 combo for duplicated.Dups
-duplicated.Dups <- function(x, incomparables=FALSE, ...)
-    .duplicated.Dups(x, incomparables=incomparables, ...)
-setMethod("duplicated", "Dups", duplicated.Dups)
+setMethod("duplicated", "Dups", .duplicated.Dups)
 
 ### For Dups objects only.
 setMethod("show", "Dups",
@@ -420,7 +405,7 @@ Dups <- function(high2low=integer())
 ### 'high2low(x)' can be *conceptually* defined with:
 ###
 ###   high2low <- function(x)
-###       sapply(seq_len(length(x)),
+###       sapply(seq_along(x),
 ###              function(i) match(x[i], x[seq_len(i-1L)]))
 ###
 ### Of course this is *very* inefficient (quadratic in time), its only value
@@ -439,7 +424,7 @@ setMethod("high2low", "vector",
     {
         ## Author: Harris A. Jaffee
         ans <- match(x, x)
-        ans[ans == seq_len(length(x))] <- NA_integer_
+        ans[ans == seq_along(x)] <- NA_integer_
         return(ans)
     }
 )
@@ -470,28 +455,53 @@ setMethod("high2low", "Vector",
 )
 
 
+### -------------------------------------------------------------------------
+### GroupingRanges objects
+### ----------------------
+###
+### A GroupingRanges object represents a "block-grouping", that is, a
+### grouping where each group is a block of adjacent elements in the original
+### collection of objects. GroupingRanges objects support the Ranges API (e.g.
+### start/end/width) in addition to the Grouping API.
+###
+
+setClass("GroupingRanges",
+    ## We put Ranges before Grouping so GroupingRanges objects inherit the
+    ## "show" method for Ranges objects instead of the method for Grouping
+    ## objects.
+    contains=c("Ranges", "Grouping"),
+    representation("VIRTUAL")
+)
+
+### Overwrite default method with optimized method for GroupingRanges objects.
+setMethod("grouplengths", "GroupingRanges",
+    function(x, i=NULL)
+    {
+        x_width <- width(x)
+        .subset_by_integer(x_width, i)
+    }
+)
+
+setClass("GroupingIRanges", contains=c("IRanges", "GroupingRanges"))
+
+
 ### -------------------------------------------------------------------------
 ### Partitioning objects
 ### --------------------
 ###
-### A Partitioning container represents a block-grouping i.e. a grouping
-### where each group contains objects that are neighbors in the original
-### collection of objects. More formally, a grouping 'x' is a block-grouping
-### iff 'togroup(x)' is sorted in increasing order (not necessarily strictly
-### increasing). In addition, a Partitioning object can be seen (and
-### manipulated) as a Ranges object where all the ranges are adjacent
-### starting at 1 (i.e. it covers an integer interval starting at 1
-### and with no overlap between the ranges). Therefore the "start/end/width"
-### API is implemented on Partitioning objects (in addition to the Grouping
-### and ManyToOneGrouping APIs).
+### A Partitioning object is a GroupingRanges object where the blocks fully
+### cover the original collection of objects and don't overlap. This makes
+### them many-to-one groupings. Furthermore, the blocks must be ordered by
+### ascending position on the original collection of objects.
+### Note that for a Partitioning object 'x', 'togroup(x)' is sorted in
+### increasing order (not necessarily strictly increasing).
+###
+### The Partitioning class is virtual with 2 concrete direct subclasses:
+### PartitioningByEnd and PartitioningByWidth.
 ###
-### The Partitioning class is virtual with 3 concrete subclasses:
-### PartitioningByEnd and PartitioningByWidth and PartitioningMap.
-### Note that we put Ranges before ManyToOneGrouping in order to have
-### Partitioning objects inherit the "show" method for Ranges objects.
 
 setClass("Partitioning",
-    contains=c("Ranges", "ManyToOneGrouping"),
+    contains=c("GroupingRanges", "ManyToOneGrouping"),
     representation(
         "VIRTUAL",
         NAMES="characterORNULL"  # R doesn't like @names !!
@@ -510,36 +520,28 @@ setMethod("getListElement", "Partitioning",
         i <- normalizeDoubleBracketSubscript(i, x, exact=exact,
                                              error.if.nomatch=TRUE)
         ## The purpose of the code below is to extract 'start(x)[i] - 1'
-        ## (stored in 'ans_shift') and 'width(x)[i]' (stored in 'ans_length')
+        ## (stored in 'ans_shift') and 'width(x)[i]' (stored in 'ans_len')
         ## in the fastest possible way. Looks like a convoluted way to
         ## extract those 2 values but it is actually 1000x faster than the
         ## naive way.
         ans_shift <- 0L
-        ans_length <- end(x)[i]
+        ans_len <- end(x)[i]
         if (i >= 2L) {
             ans_shift <- end(x)[i - 1L]
-            ans_length <- ans_length - ans_shift
+            ans_len <- ans_len - ans_shift
         }
-        seq_len(ans_length) + ans_shift
+        seq_len(ans_len) + ans_shift
     }
 )
 
-### Should be more efficient than the default method for ManyToOneGrouping
-### objects.
-setMethod("grouplength", "Partitioning",
-    function(x, i=NULL)
+### Overwrite method for ManyToOneGrouping objects with optimized method for
+### Partitioning objects.
+setMethod("togroup", "Partitioning",
+    function(x, j=NULL)
     {
         x_width <- width(x)
-        if (is.null(i))
-            return(x_width)
-        if (!is.numeric(i))
-            stop(wmsg("subscript 'i' must be a vector of integers or NULL"))
-        if (!is.integer(i))
-            i <- as.integer(i)
-        bound <- length(x_width)
-        if (S4Vectors:::anyMissingOrOutside(i, -bound, bound))
-            stop(wmsg("subscript 'i' contains NAs or out of bounds indices"))
-        x_width[i]
+        x_togroup <- rep.int(seq_along(x_width), x_width)
+        .subset_by_integer(x_togroup, j)
     }
 )
 
@@ -560,7 +562,6 @@ setReplaceMethod("names", "Partitioning", set_IRanges_names)
 
 setValidity2("Partitioning", .valid.Partitioning)
 
-
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### PartitioningByEnd uses a compact internal representation that allows
 ### fast mapping from groups to objects. However, it is not efficient for
@@ -579,16 +580,14 @@ setClass("PartitioningByEnd",
 
 setMethod("end", "PartitioningByEnd", function(x) x@end)
 
+### Overwrite method for Ranges objects with optimized method for
+### PartitioningByEnd objects.
 setMethod("length", "PartitioningByEnd", function(x) length(end(x)))
 
+### Overwrite method for ManyToOneGrouping objects with optimized method for
+### PartitioningByEnd objects.
 setMethod("nobj", "PartitioningByEnd",
-    function(x)
-    {
-        x_end <- end(x)
-        if (length(x_end) == 0L)
-            return(0L)
-        x_end[length(x_end)]
-    }
+    function(x) S4Vectors:::last_or(end(x), 0L)
 )
 
 setMethod("start", "PartitioningByEnd",
@@ -695,7 +694,7 @@ PartitioningByEnd <- function(x=integer(0), NG=NULL, names=NULL)
             return(x)
         }
         x_names <- names(x)
-        ans_end <- cumsum(elementLengths(x))
+        ans_end <- cumsum(elementNROWS(x))
     } else {
         if (!is.numeric(x))
             stop(wmsg("'x' must be either a list-like object or ",
@@ -740,20 +739,8 @@ setClass("PartitioningByWidth",
 
 setMethod("width", "PartitioningByWidth", function(x) x@width)
 
-setMethod("length", "PartitioningByWidth", function(x) length(width(x)))
-
 setMethod("end", "PartitioningByWidth", function(x) cumsum(width(x)))
 
-setMethod("nobj", "PartitioningByWidth",
-    function(x)
-    {
-        x_end <- end(x)
-        if (length(x_end) == 0L)
-            return(0L)
-        x_end[length(x_end)]
-    }
-)
-
 setMethod("start", "PartitioningByWidth",
     function(x)
     {
@@ -828,7 +815,7 @@ PartitioningByWidth <- function(x=integer(0), NG=NULL, names=NULL)
             return(x)
         }
         x_names <- names(x)
-        ans_width <- elementLengths(x)
+        ans_width <- elementNROWS(x)
     } else {
         if (!is.numeric(x))
             stop(wmsg("'x' must be either a list-like object or ",
@@ -916,9 +903,9 @@ setMethod("show", "PartitioningMap",
 ### findOverlaps()
 ###
 
-### A simple findOverlaps method that doesn't use IntervalTree but works only
+### A simple findOverlaps method that doesn't use NCList but works only
 ### on a subject with *adjacent* ranges sorted non-decreasingly.
-### Can be 30% faster or more than the real findOverlaps() (IntervalTree-based)
+### Can be 30% faster or more than the real findOverlaps() (NCList-based)
 ### when 'query' is such that 'start(query)' and 'end(query)' are also sorted
 ### non-decreasingly (which is the case if for example 'query' is a
 ### Partitioning object).
@@ -953,7 +940,7 @@ findOverlaps_Ranges_Partitioning <- function(query, subject,
     vec <- c(0L, s_end) + 0.5
     q_start2subject <- findInterval(q_start, vec)
     q_end2subject <- findInterval(q_end, vec)
-    q_hits <- rep.int(seq_len(length(q_start)),
+    q_hits <- rep.int(seq_along(q_start),
                       q_end2subject - q_start2subject + 1L)
     s_hits <- S4Vectors:::mseq(q_start2subject, q_end2subject)
     ## If 'query' is a Partitioning object, all hits are guaranteed to be
@@ -971,6 +958,25 @@ findOverlaps_Ranges_Partitioning <- function(query, subject,
         s_hits <- s_idx[s_hits]
 
     ## Make and return Hits object.
-    Hits(q_hits, s_hits, q_len, s_len)
-}   
+    Hits(q_hits, s_hits, q_len, s_len, sort.by.query=TRUE)
+}
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Old stuff (deprecated & defunct)
+###
+
+setMethod("togroup", "ANY",
+    function(x, j=NULL)
+    {
+        msg <- wmsg(
+            "Using togroup() on a ", class(x), " object is deprecated. ",
+            "Please use togroup(PartitioningByWidth(...)) instead."
+        )
+        .Deprecated(msg=msg)
+        togroup(PartitioningByWidth(x), j=j)
+    }
+)
+
+grouplength <- function(...) {.Deprecated("grouplengths"); grouplengths(...)}
 
