@@ -12,7 +12,7 @@
 ## a SplitDataFrameList. The RangesList is uncompressed, because
 ## users will likely want to apply over each Ranges separately, as
 ## they are usually in separate spaces. Also, it is difficult to
-## compress RangesLists, as lists containing Views or IntervalTrees
+## compress RangesLists, as lists containing Views or NCLists
 ## are uncompressible. The SplitDataFrameList should be compressed,
 ## because it's cheap to create from a split factor and, more
 ## importantly, cheap to get and set columns along the entire dataset,
@@ -34,8 +34,10 @@ setMethod("values", "RangedData", function(x) x@values)
 setReplaceMethod("values", "RangedData",
                  function(x, value) {
                    if (extends(class(value), "SplitDataFrameList")) {
-                     if (!identical(elementLengths(values(x)), elementLengths(value)))
-                       stop("'value' must have same elementLengths as current 'values'")
+                     if (!identical(elementNROWS(values(x)),
+                                    elementNROWS(value)))
+                       stop("'value' must have same elementNROWS ",
+                            "as current 'values'")
                    } else if (extends(class(value), "DataFrame")) {
                      value <- split(value, space(x))
                    } else {
@@ -104,7 +106,7 @@ setReplaceMethod("names", "RangedData",
                    names(x@values) <- value
                    x
                  })
-setMethod("elementLengths", "RangedData", function(x) elementLengths(ranges(x)))
+setMethod("elementNROWS", "RangedData", function(x) elementNROWS(ranges(x)))
 
 setMethod("space", "RangedData", function(x) space(ranges(x)))
 setMethod("universe", "RangedData", function(x) universe(ranges(x)))
@@ -165,7 +167,7 @@ setReplaceMethod("rownames", "RangedData",
                      } else {
                        if (!is.character(value))
                          value <- as.character(value)
-                       ends <- cumsum(elementLengths(x))
+                       ends <- cumsum(elementNROWS(x))
                        value <-
                          new("CompressedCharacterList",
                              unlistData = value,
@@ -244,8 +246,8 @@ RangedData <- function(ranges = IRanges(), ..., space = NULL, universe = NULL)
       names(ranges) <- as.character(seq_len(length(ranges)))
     space <-
       Rle(factor(names(ranges), levels = names(ranges)),
-          elementLengths(ranges))
-    N <- sum(elementLengths(ranges))
+          elementNROWS(ranges))
+    N <- sum(elementNROWS(ranges))
     NAMES <- unlist(lapply(ranges, names), use.names=FALSE)
   } else {
     if (!is(ranges, "Ranges")) {
@@ -294,7 +296,7 @@ RangedData <- function(ranges = IRanges(), ..., space = NULL, universe = NULL)
     }
   }
   else
-    values <- new2("DataFrame", nrows = N, check=FALSE)
+    values <- S4Vectors:::make_zero_col_DataFrame(N)
   if (N != nrow(values)) {
     if (nrow(values) > N)
       stop("length of value(s) in '...' greater than length of 'ranges'")
@@ -372,7 +374,7 @@ setReplaceMethod("[[", "RangedData",
                        else
                          value <- rep(value, length.out = nrx)
                      }
-                     nrows <- elementLengths(values(x))
+                     nrows <- elementNROWS(values(x))
                      inds <- seq_len(length(x))
                      spaces <- factor(rep.int(inds, nrows), inds)
                      values <- unlist(values(x), use.names=FALSE)
@@ -440,14 +442,14 @@ setMethod("[", "RangedData",
                   stop("subsetting a RangedData object ",
                        "by a RangesList subscript is not supported")
                 if (is(i, "LogicalList")) {
-                  xeltlen <- elementLengths(ranges(x))
-                  whichRep <- which(xeltlen != elementLengths(i))
+                  x_eltNROWS <- elementNROWS(ranges(x))
+                  whichRep <- which(x_eltNROWS != elementNROWS(i))
                   for (k in whichRep)
-                    i[[k]] <- rep(i[[k]], length.out = xeltlen[k])
+                    i[[k]] <- rep(i[[k]], length.out = x_eltNROWS[k])
                   i <- unlist(i, use.names=FALSE)
                 } else if (is(i, "IntegerList")) {
                   itemp <-
-                    LogicalList(lapply(elementLengths(ranges(x)), rep,
+                    LogicalList(lapply(elementNROWS(ranges(x)), rep,
                                        x = FALSE))
                   for (k in seq_len(length(itemp)))
                     itemp[[k]][i[[k]]] <- TRUE
@@ -460,7 +462,7 @@ setMethod("[", "RangedData",
                   i <- setdiff(seq(nrow(x)), -i)
                 if (is.logical(i)) {
                   igroup <-
-                    factor(rep.int(seq_len(length(x)), elementLengths(x)),
+                    factor(rep.int(seq_len(length(x)), elementNROWS(x)),
                            levels = seq_len(length(x)))
                   if (length(i) < nrow(x))
                     i <- rep(i, length.out = nrow(x))
@@ -476,7 +478,7 @@ setMethod("[", "RangedData",
                     if (S4Vectors:::anyMissing(i)) ## cannot subset by NAs yet
                       stop("invalid rownames specified")
                   }
-                  starts <- cumsum(c(1L, head(elementLengths(x), -1)))
+                  starts <- cumsum(c(1L, head(elementNROWS(x), -1)))
                   igroup <-
                     factor(findInterval(i, starts), levels = seq_len(length(x)))
                   if (anyDuplicated(runValue(Rle(igroup))))
@@ -488,7 +490,7 @@ setMethod("[", "RangedData",
                 ranges <- S4Vectors:::subset_List_by_List(ranges, isplit)
                 values <- S4Vectors:::subset_List_by_List(values, isplit)
                 if (drop) {
-                  ok <- (elementLengths(ranges) > 0)
+                  ok <- (elementNROWS(ranges) > 0)
                   ranges <- ranges[ok]
                   values <- values[ok]
                 }
@@ -535,7 +537,16 @@ setMethod("c", "RangedData", function(x, ..., recursive = FALSE) {
   rd
 })
 
+.split_RangedData_deprecation_msg <- wmsg(
+    "The \"split\" method for RangedData objects is deprecated. ",
+    "Please replace the RangedData object with a GRanges object ",
+    "and split it. This will produce a GRangesList instead of a ",
+    "RangedDataList object. (The GRanges and GRangesList classes ",
+    "are defined in the GenomicRanges package.)"
+)
+
 setMethod("split", "RangedData", function(x, f, drop = FALSE) {
+  .Deprecated(msg=.split_RangedData_deprecation_msg)
   if (length(f) > nrow(x) || nrow(x) %% length(f) > 0)
     stop("nrow(x) is not a multiple of length(f)")
   splitInd <- split(seq_len(nrow(x)), f, drop)
@@ -592,7 +603,7 @@ setMethod("rbind", "RangedData", function(..., deparse.level=1) {
         spaceLabels <- as.character(spaceLevels)
     }
     data.frame(space =
-               factor(rep.int(seq_len(length(x)), elementLengths(x)),
+               factor(rep.int(seq_len(length(x)), elementNROWS(x)),
                       levels = spaceLevels,
                       labels = spaceLabels),
                as.data.frame(unlist(x, use.names = FALSE)),
@@ -613,8 +624,7 @@ setMethod("rbind", "RangedData", function(..., deparse.level=1) {
     as.data.frame(stacked, row.names = row.names, optional = optional)
 }
 
-### S3/S4 combo for as.data.frame.RangedData
-as.data.frame.RangedData <- function(x, row.names=NULL, optional=FALSE, ...)
+.as.data.frame.RangedData <- function(x, row.names=NULL, optional=FALSE, ...)
 {
     if (!(is.null(row.names) || is.character(row.names)))
         stop("'row.names'  must be NULL or a character vector")
@@ -625,7 +635,7 @@ as.data.frame.RangedData <- function(x, row.names=NULL, optional=FALSE, ...)
                row.names = row.names,
                stringsAsFactors = FALSE)
 }
-setMethod("as.data.frame", "RangedData",as.data.frame.RangedData)
+setMethod("as.data.frame", "RangedData", .as.data.frame.RangedData)
 
 setAs("RangedData", "DataFrame",
       function(from)
@@ -668,7 +678,7 @@ setAs("RleList", "RangedData",
 
 setAs("RleViewsList", "RangedData", function(from) {
   subject <- subject(from)
-  from_ranges <- restrict(ranges(from), 1L, elementLengths(subject),
+  from_ranges <- restrict(ranges(from), 1L, elementNROWS(subject),
                           keep.all.ranges = TRUE)
 ### FIXME: do we want to insert NAs for out of bounds views?
   score <- subject[from_ranges]
@@ -712,9 +722,8 @@ setAs("RangesList", "RangedData",
                         "'from' (accessed with 'mcols(unlist(from))') ",
                         "to the data columns (aka values) of the returned ",
                         "RangedData object")
-            unlisted_values <- new2("DataFrame",
-                                    nrows=length(unlisted_from),
-                                    check=FALSE)
+            unlisted_values <-
+                S4Vectors:::make_zero_col_DataFrame(length(unlisted_from))
         }
         ans_values <- newCompressedList0("CompressedSplitDataFrameList",
                                          unlisted_values,
@@ -860,14 +869,27 @@ setClass("RangedDataList",
          prototype = prototype(elementType = "RangedData"),
          contains = "SimpleList")
 
+.RangedDataList_deprecation_msg <- wmsg(
+    "RangedDataList objects are deprecated in favor of GRangesList objects ",
+    "(the GRangesList class is defined in the GenomicRanges package)."
+)
+
 RangedDataList <- function(...)
 {
+
+  .Deprecated(msg=.RangedDataList_deprecation_msg)
   listData <- list(...)
   if (length(listData) == 1 && is.list(listData[[1L]]))
     listData <- listData[[1L]]
   S4Vectors:::new_SimpleList_from_list("RangedDataList", listData)
 }
 
+setMethod("show", "RangedDataList",
+          function(object) {
+            .Deprecated(msg=.RangedDataList_deprecation_msg)
+            callNextMethod()
+          })
+
 setMethod("unlist", "RangedDataList",
           function(x, recursive = TRUE, use.names = TRUE) {
             if (!identical(recursive, TRUE))
