@@ -4,23 +4,23 @@
 ###
 
 
-setMethod("compareRecursively", "Ranges", function(x) FALSE)
+setMethod("pcompareRecursively", "Ranges", function(x) FALSE)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### compare()
+### pcompare()
 ###
 ### Ranges are ordered by starting position first and then by width.
 ### This way, the space of ranges is totally ordered.
-### This "compare" method returns one of the 13 predefined codes (>= -6 and
+### This "pcompare" method returns one of the 13 predefined codes (>= -6 and
 ### <= 6) described in the man page. The signs of those codes reflect this
 ### order.
 ###
 
-setMethod("compare", c("Ranges", "Ranges"),
+setMethod("pcompare", c("Ranges", "Ranges"),
     function(x, y)
     {
-        .Call2("Ranges_compare",
+        .Call2("Ranges_pcompare",
                start(x), width(x), start(y), width(y),
                PACKAGE="IRanges")
     }
@@ -51,9 +51,9 @@ setMethod("match", c("Ranges", "Ranges"),
         ## Equivalent to (but faster than):
         ##     findOverlaps(x, table, type="equal", select="first")
         ## except when 'x' or 'table' contain empty ranges.
-        S4Vectors:::matchIntegerPairs(start(x), width(x),
-                                      start(table), width(table),
-                                      nomatch=nomatch, method=method)
+        matchIntegerPairs(start(x), width(x),
+                          start(table), width(table),
+                          nomatch=nomatch, method=method)
     }
 )
 
@@ -64,7 +64,7 @@ setMethod("match", c("Ranges", "Ranges"),
 
 setMethod("selfmatch", "Ranges",
     function(x, method=c("auto", "quick", "hash"))
-        S4Vectors:::selfmatchIntegerPairs(start(x), width(x), method=method)
+        selfmatchIntegerPairs(start(x), width(x), method=method)
 )
 
 
@@ -72,12 +72,13 @@ setMethod("selfmatch", "Ranges",
 ### Ordering ranges
 ###
 ### order(), sort(), rank() on Ranges objects are consistent with the order
-### on ranges implied by compare().
+### on ranges implied by pcompare().
 ###
 
 ### 'na.last' is pointless (Ranges objects don't contain NAs) so is ignored.
+### 'method' is also ignored at the moment.
 setMethod("order", "Ranges",
-    function(..., na.last=TRUE, decreasing=FALSE)
+    function(..., na.last=TRUE, decreasing=FALSE, method=c("shell", "radix"))
     {
         if (!isTRUEorFALSE(decreasing))
             stop("'decreasing' must be TRUE or FALSE")
@@ -85,8 +86,8 @@ setMethod("order", "Ranges",
         args <- list(...)
         if (length(args) == 1L) {
             x <- args[[1L]]
-            return(S4Vectors:::orderIntegerPairs(start(x), width(x),
-                                                 decreasing=decreasing))
+            return(orderIntegerPairs(start(x), width(x),
+                                     decreasing=decreasing))
         }
         order_args <- vector("list", 2L * length(args))
         idx <- 2L * seq_along(args)
