@@ -4,23 +4,42 @@
 ###
 
 
+### TODO: We need a ranges() setter for Views objects that provides this
+### functionality. Once we have it, use it instead of this.
+.set_Views_ranges <- function(x, new_ranges)
+{
+    new_mcols <- mcols(new_ranges)
+    mcols(new_ranges) <- NULL
+    BiocGenerics:::replaceSlots(x, ranges=new_ranges,
+                                   elementMetadata=new_mcols,
+                                   check=FALSE)
+}
+
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### range()
 ###
 
+### Always return an IRanges (or NormalIRanges) *instance* whatever Ranges
+### derivative the input is, so does NOT act like an endomorphism in general. 
 setMethod("range", "Ranges",
     function(x, ..., na.rm=FALSE)
     {
         if (!identical(na.rm, FALSE))
-            warning("'na.rm' argument is ignored")
+            warning("\"range\" method for Ranges objects ",
+                    "ignores the 'na.rm' argument")
         args <- unname(list(x, ...))
-        if (length(args) == 0L)  # should never happen
-            return(IRanges())
-        if (!all(sapply(args, is, "Ranges")))
-            stop("all arguments in '...' must be Ranges objects")
-        xx <- do.call(c, lapply(args, as, "IRanges"))
-        y <- .Call2("IRanges_range", xx, PACKAGE="IRanges")
-        as(y, class(args[[1L]]))
+        ## TODO: Replace line below with
+        ##     args <- lapply(args, ranges)
+        ## when ranges() works on Ranges objects.
+        args <- lapply(args,
+                       function(arg) IRanges(start(arg), width=width(arg)))
+        ir <- do.call(c, args)
+
+        ans <- .Call2("IRanges_range", ir, PACKAGE="IRanges")
+        if (is(x, "NormalIRanges"))
+            ans <- as(ans, "NormalIRanges")
+        ans
     }
 )
 
@@ -51,11 +70,11 @@ setMethod("range", "RangesList",
     ans_unlistData <- IRanges(viewMins(sv)[is_not_empty_view],
                               viewMaxs(ev)[is_not_empty_view])
     ans_partitioning <- new2("PartitioningByEnd",
-                             end=cumsum(is_not_empty_view), check=FALSE)
-    ans <- new2("CompressedIRangesList",
-                unlistData=ans_unlistData,
-                partitioning=ans_partitioning,
-                check=FALSE)
+                                         end=cumsum(is_not_empty_view),
+                                         check=FALSE)
+    ans <- new2("CompressedIRangesList", unlistData=ans_unlistData,
+                                         partitioning=ans_partitioning,
+                                         check=FALSE)
     names(ans) <- names(x)
     mcols(ans) <- mcols(x)
     ans
@@ -70,11 +89,6 @@ setMethod("range", "CompressedIRangesList",
     }
 )
 
-setMethod("range", "IntervalForest",
-          function(x, ..., na.rm=FALSE)
-            as(range(as(x, "CompressedIRangesList"), ..., na.rm = na.rm),
-               "IntervalForest"))
-
 setMethod("range", "RangedData", function(x, ..., na.rm) {
   args <- list(x, ...)
   rangeLists <- lapply(args, ranges)
@@ -90,7 +104,9 @@ setGeneric("reduce", signature="x",
     function(x, ...) standardGeneric("reduce")
 )
 
-setMethod("reduce", "IRanges",
+### Always return an IRanges (or NormalIRanges) *instance* whatever Ranges
+### derivative the input is, so does NOT act like an endomorphism in general. 
+setMethod("reduce", "Ranges",
     function(x, drop.empty.ranges=FALSE, min.gapwidth=1L,
                 with.revmap=FALSE,
                 with.inframe.attrib=FALSE)
@@ -113,46 +129,36 @@ setMethod("reduce", "IRanges",
                         with.revmap,
                         with.inframe.attrib,
                         PACKAGE="IRanges")
-        ans <- unsafe.update(x, start=C_ans$start, width=C_ans$width,
-                                names=NULL)
+        ans <- new2("IRanges", start=C_ans$start,
+                               width=C_ans$width,
+                               check=FALSE)
+        if (is(x, "NormalIRanges"))
+            ans <- as(ans, "NormalIRanges")
         if (with.revmap) {
             revmap <- IntegerList(C_ans$revmap)
             mcols(ans) <- DataFrame(revmap=revmap)
         }
         if (with.inframe.attrib) {
             inframe <- new2("IRanges", start=C_ans$inframe.start,
-                                       width=width(x), check=FALSE)
+                                       width=width(x),
+                                       check=FALSE)
             attr(ans, "inframe") <- inframe
         }
         ans
     }
 )
 
-setMethod("reduce", "Ranges",
-    function(x, drop.empty.ranges=FALSE, min.gapwidth=1L,
-                with.revmap=FALSE, 
-                with.inframe.attrib=FALSE)
-    {
-        ir <- as(x, "IRanges")
-        y <- reduce(ir, drop.empty.ranges=drop.empty.ranges,
-                        min.gapwidth=min.gapwidth,
-                        with.revmap=with.revmap,
-                        with.inframe.attrib=with.inframe.attrib)
-        as(y, class(x))
-    }
-)
-
 setMethod("reduce", "Views",
     function(x, drop.empty.ranges=FALSE, min.gapwidth=1L,
                 with.revmap=FALSE,
                 with.inframe.attrib=FALSE)
     {
-        x@ranges <- reduce(ranges(x),
-                           drop.empty.ranges=drop.empty.ranges,
-                           min.gapwidth=min.gapwidth,
-                           with.revmap=with.revmap,
-                           with.inframe.attrib=with.inframe.attrib)
-        x
+        new_ranges <- callGeneric(x@ranges,
+                                  drop.empty.ranges=drop.empty.ranges,
+                                  min.gapwidth=min.gapwidth,
+                                  with.revmap=with.revmap,
+                                  with.inframe.attrib=with.inframe.attrib)
+        .set_Views_ranges(x, new_ranges)
     }
 )
 
@@ -160,10 +166,13 @@ setMethod("reduce", "RangesList",
     function(x, drop.empty.ranges=FALSE, min.gapwidth=1L,
                 with.revmap=FALSE,
                 with.inframe.attrib=FALSE)
+    {
         endoapply(x, reduce, drop.empty.ranges = drop.empty.ranges,
-                     min.gapwidth = min.gapwidth,
-                     with.revmap=with.revmap,
-                     with.inframe.attrib = with.inframe.attrib))
+                             min.gapwidth = min.gapwidth,
+                             with.revmap=with.revmap,
+                             with.inframe.attrib = with.inframe.attrib)
+    }
+)
 
 ### 'with.inframe.attrib' is ignored for now.
 ### TODO: Support 'with.inframe.attrib=TRUE'.
@@ -189,7 +198,8 @@ setMethod("reduce", "CompressedIRangesList",
                         x, drop.empty.ranges, min.gapwidth,
                         with.revmap,
                         PACKAGE="IRanges")
-        ans_unlistData <- new2("IRanges", start=C_ans$start, width=C_ans$width,
+        ans_unlistData <- new2("IRanges", start=C_ans$start,
+                                          width=C_ans$width,
                                           check=FALSE)
         if (with.revmap) {
             revmap <- IntegerList(C_ans$revmap)
@@ -201,23 +211,12 @@ setMethod("reduce", "CompressedIRangesList",
     }
 )
 
-setMethod("reduce", "IntervalForest",
-          function(x, drop.empty.ranges=FALSE, min.gapwidth=1L,
-                      with.inframe.attrib=FALSE)
-          {
-            if (!drop.empty.ranges)
-              stop("'drop.empty.ranges' must be TRUE in ",
-                   "'reduce,IntervalForest'")
-            as(reduce(as(x, "CompressedIRangesList"),
-                      min.gapwidth = min.gapwidth,
-                      with.inframe.attrib = with.inframe.attrib),
-               "IntervalForest")
-            })
-
 setMethod("reduce", "RangedData",
           function(x, by = character(), drop.empty.ranges=FALSE,
                    min.gapwidth=1L, with.inframe.attrib=FALSE)
           {
+            .Deprecated(msg=c("the \"reduce\" method for RangedData object ",
+                              "is deprecated"))
             if (!isTRUEorFALSE(drop.empty.ranges))
                 stop("'drop.empty.ranges' must be TRUE or FALSE")
             if (!isSingleNumber(min.gapwidth))
@@ -259,9 +258,9 @@ setMethod("reduce", "RangedData",
                        drop.empty.ranges = drop.empty.ranges,
                        min.gapwidth = min.gapwidth,
                        with.inframe.attrib = with.inframe.attrib)
-              listData <- new2("DataFrame", nrows=sum(elementLengths(ranges)),
-                               check=FALSE)
-              end <- cumsum(elementLengths(ranges))
+              listData <-
+                S4Vectors:::make_zero_col_DataFrame(sum(elementNROWS(ranges)))
+              end <- cumsum(elementNROWS(ranges))
               names(end) <- names(ranges)
               partitioning <- PartitioningByEnd(end)
               initialize(x,
@@ -281,7 +280,9 @@ setGeneric("gaps", signature="x",
     function(x, start=NA, end=NA) standardGeneric("gaps")
 )
 
-setMethod("gaps", "IRanges",
+### Always return an IRanges (or NormalIRanges) *instance* whatever Ranges
+### derivative the input is, so does NOT act like an endomorphism in general. 
+setMethod("gaps", "Ranges",
     function(x, start=NA, end=NA)
     {
         start <- S4Vectors:::normargSingleStartOrNA(start)
@@ -289,17 +290,12 @@ setMethod("gaps", "IRanges",
         C_ans <- .Call2("IRanges_gaps",
                         start(x), width(x), start, end,
                         PACKAGE="IRanges")
-        initialize(x, start=C_ans$start, width=C_ans$width, NAMES=NULL,
-                      elementMetadata=NULL)
-    }
-)
-
-setMethod("gaps", "Ranges",
-    function(x, start=NA, end=NA)
-    {
-        ir <- as(x, "IRanges")
-        y <- gaps(ir, start=start, end=end)
-        as(y, class(x))
+        ans <- new2("IRanges", start=C_ans$start,
+                               width=C_ans$width,
+                               check=FALSE)
+        if (is(x, "NormalIRanges"))
+            ans <- as(ans, "NormalIRanges")
+        ans
     }
 )
 
@@ -318,52 +314,49 @@ setMethod("gaps", "Views",
             start <- 1L
         if (is.na(end))
             end <- length(subject(x))
-        x@ranges <- gaps(ranges(x), start=start, end=end)
-        x
+        new_ranges <- gaps(x@ranges, start=start, end=end)
+        .set_Views_ranges(x, new_ranges)
     }
 )
 
 setMethod("gaps", "RangesList",
-          function(x, start = NA, end = NA)
-          {
-              lx <- length(x)
-              if (!S4Vectors:::isNumericOrNAs(start))
-                  stop("'start' must be an integer vector or NA")
-              if (!is.integer(start))
-                  start <- as.integer(start)
-              if (!S4Vectors:::isNumericOrNAs(end))
-                  stop("'end' must be an integer vector or NA")
-              if (!is.integer(end))
-                  end <- as.integer(end)
-              start <- IntegerList(as.list(S4Vectors:::recycleVector(start, lx)))
-              end <- IntegerList(as.list(S4Vectors:::recycleVector(end, lx)))
-              mendoapply(gaps, x, start = start, end = end)
-          })
+    function(x, start = NA, end = NA)
+    {
+        lx <- length(x)
+        if (!S4Vectors:::isNumericOrNAs(start))
+            stop("'start' must be an integer vector or NA")
+        if (!is.integer(start))
+            start <- as.integer(start)
+        if (!S4Vectors:::isNumericOrNAs(end))
+            stop("'end' must be an integer vector or NA")
+        if (!is.integer(end))
+            end <- as.integer(end)
+        start <- IntegerList(as.list(S4Vectors:::recycleVector(start, lx)))
+        end <- IntegerList(as.list(S4Vectors:::recycleVector(end, lx)))
+        mendoapply(gaps, x, start = start, end = end)
+    }
+)
 
 setMethod("gaps", "CompressedIRangesList",
-          function(x, start = NA, end = NA)
-          {
-              lx <- length(x)
-              if (!S4Vectors:::isNumericOrNAs(start))
-                  stop("'start' must be an integer vector or NA")
-              if (!is.integer(start))
-                  start <- as.integer(start)
-              if (!S4Vectors:::isNumericOrNAs(end))
-                  stop("'end' must be an integer vector or NA")
-              if (!is.integer(end))
-                  end <- as.integer(end)
-              if ((length(start) != 1) || (length(end) != 1)) {
-                  start <- S4Vectors:::recycleVector(start, lx)
-                  end <- S4Vectors:::recycleVector(end, lx)
-              }
-              .Call2("CompressedIRangesList_gaps", x, start, end,
-                    PACKAGE="IRanges")
-          })
-
-setMethod("gaps", "IntervalForest",
-          function(x, start = NA, end = NA)
-            as(gaps(as(x, "CompressedIRangesList"), start = start, end = end),
-               "IntervalForest"))
+    function(x, start = NA, end = NA)
+    {
+        lx <- length(x)
+        if (!S4Vectors:::isNumericOrNAs(start))
+            stop("'start' must be an integer vector or NA")
+        if (!is.integer(start))
+            start <- as.integer(start)
+        if (!S4Vectors:::isNumericOrNAs(end))
+            stop("'end' must be an integer vector or NA")
+        if (!is.integer(end))
+            end <- as.integer(end)
+        if ((length(start) != 1) || (length(end) != 1)) {
+            start <- S4Vectors:::recycleVector(start, lx)
+            end <- S4Vectors:::recycleVector(end, lx)
+        }
+        .Call2("CompressedIRangesList_gaps", x, start, end,
+               PACKAGE="IRanges")
+    }
+)
 
 ### 'start' and 'end' are ignored.
 setMethod("gaps", "MaskCollection",
@@ -387,6 +380,8 @@ setMethod("gaps", "MaskCollection",
 
 setGeneric("disjoin", function(x, ...) standardGeneric("disjoin"))
 
+### Always return an IRanges *instance* whatever Ranges derivative the input
+### is, so does NOT act like an endomorphism in general. 
 setMethod("disjoin", "Ranges",
     function(x)
     {
@@ -394,14 +389,21 @@ setMethod("disjoin", "Ranges",
         ## ends: original ends and start-1 when inside another interval
         starts <- unique(start(x))
         ends <- unique(end(x))
-        adj_start <- sort(unique(c(starts, ends + 1L)))
-        adj_end <- sort(unique(c(ends, starts - 1L)))
-        adj <- update(x, start=head(adj_start, -1L), end=tail(adj_end, -1L),
-                      names=NULL, check=FALSE)
+        adj_start <- head(sort(unique(c(starts, ends + 1L))), -1L)
+        adj_end <- tail(sort(unique(c(ends, starts - 1L))), -1L)
+        adj_width <- adj_end - adj_start + 1L
+        adj <- new2("IRanges", start=adj_start,
+                               width=adj_width,
+                               check=FALSE)
         subsetByOverlaps(adj,  x)
     }
 )
 
+### Basically a no-op but returns a NormalIRanges *instance* for consistency
+### with how the other inter-range transformations (ranges(), reduce(), gaps())
+### behave on a NormalIRanges object.
+setMethod("disjoin", "NormalIRanges", function(x) as(x, "NormalIRanges"))
+
 setMethod("disjoin", "RangesList", function(x) endoapply(x, disjoin))
 
 setMethod("disjoin", "CompressedIRangesList",
@@ -411,7 +413,7 @@ setMethod("disjoin", "CompressedIRangesList",
                   ## unlist CompressedIntegerList, with integer(0) as 0
               {
                   w <- integer(length(x))
-                  w[elementLengths(x) != 0L] <- unlist(x, use.names=FALSE)
+                  w[elementNROWS(x) != 0L] <- unlist(x, use.names=FALSE)
                   w
               }
 
@@ -430,7 +432,7 @@ setMethod("disjoin", "CompressedIRangesList",
                   lvls <- seq_along(x)
               d <- disjoin(local, ...)
               vec <- unlist(start(shift(rng, offset)), use.names=FALSE)
-              lvls0 <- lvls[elementLengths(rng) != 0]
+              lvls0 <- lvls[elementNROWS(rng) != 0]
               f <- lvls0[findInterval(start(d), vec)]
               d <- split(d, factor(f, levels=lvls))
               if (is.null(names(x)))
@@ -440,9 +442,33 @@ setMethod("disjoin", "CompressedIRangesList",
               shift(d, -offset)
           })
 
-setMethod("disjoin", "IntervalForest",
-          function(x, ...)
-            as(disjoin(as(x, "CompressedIRangesList"), ...), "IntervalForest"))
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### isDisjoint()
+###
+
+setGeneric("isDisjoint", function(x, ...) standardGeneric("isDisjoint"))
+
+setMethod("isDisjoint", "Ranges",
+    function(x)
+    {
+        x_len <- length(x)
+        if (x_len < 2L)
+            return(TRUE)
+        x_start <- start(x)
+        x_end <- end(x)
+        oo <- order(x)
+        start2 <- x_start[oo]
+        end2 <- x_end[oo]
+        all(start2[-1L] > end2[-x_len])
+    }
+)
+
+setMethod("isDisjoint", "NormalIRanges", function(x) TRUE)
+
+setMethod("isDisjoint", "RangesList",
+    function(x) unlist(lapply(x, isDisjoint))
+)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -472,6 +498,10 @@ setMethod("disjointBins", "Ranges",
     }
 )
 
+### Overwrite above method with trivial method for NormalIRanges objects.
+setMethod("disjointBins", "NormalIRanges", function(x) rep.int(1L, length(x)))
+
 setMethod("disjointBins", "RangesList",
-          function(x) as(lapply(x, disjointBins), "IntegerList"))
+    function(x) as(lapply(x, disjointBins), "IntegerList")
+)
 
