@@ -29,7 +29,7 @@
 ###
 
 setMethod("union", c("Ranges", "Ranges"),
-    function(x, y, ...)
+    function(x, y)
     {
         ## We need to downgrade 'x' to an IRanges instance 'x0' so 'c(x0, y)'
         ## is guaranteed to work (even e.g. if 'x' is a NormalIRanges object).
@@ -57,7 +57,8 @@ setMethod("union", c("CompressedIRangesList", "CompressedIRangesList"),
             if (length(y) != len)
               y <- y[S4Vectors:::recycleVector(seq_len(length(y)), len)]
             xy <- c(unlist(x, use.names = FALSE), unlist(y, use.names = FALSE))
-            xy_list <- split(xy, factor(c(togroup(x), togroup(y)),
+            xy_list <- split(xy, factor(c(togroup(PartitioningByWidth(x)),
+                                          togroup(PartitioningByWidth(y))),
                                         seq_len(length(x))))
             names(xy_list) <- names(x)
             reduce(xy_list, drop.empty.ranges=TRUE)
@@ -69,7 +70,7 @@ setMethod("union", c("CompressedIRangesList", "CompressedIRangesList"),
 ###
 
 setMethod("intersect", c("Ranges", "Ranges"),
-    function(x, y, ...)
+    function(x, y)
     {
         if (length(x) == 0L)
             return(x)
@@ -84,7 +85,7 @@ setMethod("intersect", c("RangesList", "RangesList"),
 
 setMethod("intersect", c("CompressedIRangesList", "CompressedIRangesList"),
           function(x, y) {
-            nonempty <- elementLengths(x) != 0L
+            nonempty <- elementNROWS(x) != 0L
             rx <- unlist(range(x), use.names = FALSE)
             startx <- integer()
             startx[nonempty] <- start(rx)
@@ -93,13 +94,26 @@ setMethod("intersect", c("CompressedIRangesList", "CompressedIRangesList"),
             setdiff(x, gaps(y, start = startx, end = endx))
           })
 
+setMethod("intersect", c("CompressedAtomicList", "CompressedAtomicList"),
+          function(x, y) {
+              fx <- if (!is(x, "IntegerList")) as(x, "FactorList") else x
+              fy <- if (!is(y, "IntegerList")) as(y, "FactorList") else y
+              m <- S4Vectors:::matchIntegerPairs(togroup(PartitioningByEnd(x)),
+                                                 unlist(fx, use.names=FALSE),
+                                                 togroup(PartitioningByEnd(y)),
+                                                 unlist(fy, use.names=FALSE),
+                                                 nomatch=0L)
+              m[duplicated(m)] <- 0L
+              x[relist(m > 0L, x)]
+          })
+
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### setdiff()
 ###
 
 setMethod("setdiff", c("Ranges", "Ranges"),
-    function(x, y, ...)
+    function(x, y)
     {
         if (length(x) == 0L)
             return(x)
@@ -114,7 +128,7 @@ setMethod("setdiff", c("RangesList", "RangesList"),
 
 setMethod("setdiff", c("CompressedIRangesList", "CompressedIRangesList"),
           function(x, y) {
-            nonempty <- elementLengths(x) != 0L
+            nonempty <- elementNROWS(x) != 0L
             rx <- unlist(range(x), use.names = FALSE)
             startx <- integer()
             startx[nonempty] <- start(rx)
@@ -133,7 +147,7 @@ setGeneric("punion", signature=c("x", "y"),
 )
 
 setMethod("punion", c("Ranges", "Ranges"),
-    function(x, y, fill.gap=FALSE, ...)
+    function(x, y, fill.gap=FALSE)
     {
         if (length(x) != length(y))
             stop("'x' and 'y' must have the same length")
@@ -157,6 +171,10 @@ setMethod("punion", c("Ranges", "Ranges"),
     }
 )
 
+setMethod("punion", c("Pairs", "missing"), function(x, y, ...) {
+              callGeneric(first(x), second(x), ...)
+          })
+
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### pintersect()
@@ -167,7 +185,7 @@ setGeneric("pintersect", signature=c("x", "y"),
 )
 
 setMethod("pintersect", c("Ranges", "Ranges"),
-    function(x, y, resolve.empty=c("none", "max.start", "start.x"), ...)
+    function(x, y, resolve.empty=c("none", "max.start", "start.x"))
     {
         if (length(x) != length(y))
             stop("'x' and 'y' must have the same length")
@@ -217,6 +235,10 @@ setMethod("pintersect", c("Ranges", "Ranges"),
     }
 )
 
+setMethod("pintersect", c("Pairs", "missing"), function(x, y, ...) {
+              callGeneric(first(x), second(x), ...)
+          })
+
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### psetdiff()
@@ -227,7 +249,7 @@ setGeneric("psetdiff", signature=c("x", "y"),
 )
 
 setMethod("psetdiff", c("Ranges", "Ranges"),
-    function(x, y, ...)
+    function(x, y)
     {
         if (length(x) != length(y))
             stop("'x' and 'y' must have the same length")
@@ -252,6 +274,9 @@ setMethod("psetdiff", c("Ranges", "Ranges"),
     }
 )
 
+setMethod("psetdiff", c("Pairs", "missing"), function(x, y, ...) {
+              callGeneric(first(x), second(x), ...)
+          })
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### pgap()
@@ -262,7 +287,7 @@ setGeneric("pgap", signature=c("x", "y"),
 )
 
 setMethod("pgap", c("Ranges", "Ranges"),
-    function(x, y, ...)
+    function(x, y)
     {
         if (length(x) != length(y))
             stop("'x' and 'y' must have the same length")
