@@ -22,9 +22,9 @@ findOverlaps_NCLists <- IRanges:::findOverlaps_NCLists
 ### Used in the unit tests for GNCList located in GenomicRanges.
 .make_Hits_from_q2s <- function(q2s, s_len)
 {
-    q_hits <- rep.int(seq_along(q2s), elementLengths(q2s))
+    q_hits <- rep.int(seq_along(q2s), elementNROWS(q2s))
     s_hits <- as.integer(unlist(q2s, use.names=FALSE))
-    Hits(q_hits, s_hits, length(q2s), s_len)
+    Hits(q_hits, s_hits, length(q2s), s_len, sort.by.query=TRUE)
 }
 
 .make_Hits_from_s2q <- function(s2q, q_len)
@@ -38,58 +38,78 @@ findOverlaps_NCLists <- IRanges:::findOverlaps_NCLists
 }
 
 ### Used in the unit tests for GNCList located in GenomicRanges.
-.overlap_score <- function(query, subject)
+.min_overlap_score <- function(maxgap=0L, minoverlap=1L)
 {
-    pmin(end(query), end(subject)) - pmax(start(query), start(subject)) + 1L
+    if (maxgap != 0L && minoverlap > 1L)
+        stop("'minoverlap' must be <= 1 when 'maxgap' is not 0")
+    minoverlap - maxgap
 }
 
-.get_query_overlaps <- function(query, subject, min.score, type)
+### Used in the unit tests for GNCList located in GenomicRanges.
+.overlap_score <- function(query, subject, type="any")
 {
-    ok <- .overlap_score(query, subject) >= min.score
-    if (type %in% c("start", "end")) {
-        if (type == "start")
-            d <- abs(start(subject) - start(query))
-        if (type == "end") 
-            d <- abs(end(subject) - end(query))
-        if (min.score >= 1L) {
-            dmax <- 0L
-        } else {
-            dmax <- 1L - min.score
-        }
-        ok <- ok & (d <= dmax)
-    } else if (type != "any") {
-        codes <- rangeComparisonCodeToLetter(compare(query, subject))
-        type_codes <- switch(type,
-            #"start"  = c("f", "g", "h"),
-            #"end"    = c("d", "g", "j"),
-            "within" = c("f", "g", "i", "j"),
-            "extend" = c("d", "e", "g", "h"),
-            "equal"  = "g"
-        )
-        ok <- ok & (codes %in% type_codes)
+    overlap_score <- pmin(end(query), end(subject)) -
+                     pmax(start(query), start(subject)) + 1L
+    if (type %in% c("start", "end", "equal"))
+        overlap_score <- pmax(overlap_score, 0L)
+    overlap_score
+}
+
+### Used in the unit tests for GNCList located in GenomicRanges.
+.get_query_overlaps <- function(query, subject,
+                                maxgap, min_overlap_score, type)
+{
+    ok <- .overlap_score(query, subject, type) >= min_overlap_score
+    if (type == "any")
+        return(ok)
+    if (type != "end") 
+        d1 <- abs(start(subject) - start(query))
+    if (type != "start")
+        d2 <- abs(end(subject) - end(query))
+    if (type == "start")
+        return(ok & d1 <= maxgap)
+    if (type == "end")
+        return(ok & d2 <= maxgap)
+    if (type == "equal")
+        return(ok & d1 <= maxgap & d2 <= maxgap)
+    if (type == "within") {
+        ok2 <- start(query) >= start(subject) & end(query) <= end(subject)
+    } else {  # type == "extend"
+        ok2 <- start(query) <= start(subject) & end(query) >= end(subject)
     }
-    which(ok)
+    ok <- ok & ok2
+    if (maxgap > 0L)
+        ok <- ok & (d1 + d2) <= maxgap
+    ok
 }
 
-.findOverlaps_naive <- function(query, subject, min.score=1L,
+.findOverlaps_naive <- function(query, subject,
+                                maxgap=0L, minoverlap=1L,
                                 type=c("any", "start", "end",
                                        "within", "extend", "equal"),
                                 select=c("all", "first", "last", "arbitrary",
                                          "count"))
 {
     type <- match.arg(type)
+    if (type == "any") {
+        min_overlap_score <- .min_overlap_score(maxgap, minoverlap)
+    } else {
+        min_overlap_score <- minoverlap
+    }
     select <- match.arg(select)
     hits_per_query <- lapply(seq_along(query),
-        function(i) .get_query_overlaps(query[i], subject, min.score, type))
+        function(i)
+            which(.get_query_overlaps(query[i], subject,
+                                      maxgap, min_overlap_score, type)))
     hits <- .make_Hits_from_q2s(hits_per_query, length(subject))
     selectHits(hits, select=select)
 }
 
 test_NCList <- function()
 {
-    x <- IRanges(rep.int(1:5, 5:1), c(1:5, 2:5, 3:5, 4:5, 5),
-                 names=LETTERS[1:15])
-    mcols(x) <- DataFrame(score=seq(0.7, by=0.045, length.out=15))
+    x <- IRanges(rep.int(1:6, 6:1), c(0:5, 1:5, 2:5, 3:5, 4:5, 5),
+                 names=LETTERS[1:21])
+    mcols(x) <- DataFrame(score=seq(0.7, by=0.045, length.out=21))
     nclist <- NCList(x)
 
     checkTrue(is(nclist, "NCList"))
@@ -107,7 +127,7 @@ test_NCList <- function()
 test_findOverlaps_NCList <- function()
 {
     query <- IRanges(-3:7, width=3)
-    subject <- IRanges(rep.int(1:5, 5:1), c(1:5, 2:5, 3:5, 4:5, 5))
+    subject <- IRanges(rep.int(1:6, 6:1), c(0:5, 1:5, 2:5, 3:5, 4:5, 5))
 
     target0 <- .findOverlaps_naive(query, subject)
     current <- findOverlaps_NCList(query, NCList(subject))
@@ -123,10 +143,10 @@ test_findOverlaps_NCList <- function()
         q_revperm[q_perm] <- seq_along(q_perm)
         s_revperm <- integer(length(s_perm))
         s_revperm[s_perm] <- seq_along(s_perm)
-        target <- remapHits(target0, query.map=q_revperm,
-                                     new.queryLength=length(q_perm),
-                                     subject.map=s_revperm,
-                                     new.subjectLength=length(s_perm))
+        target <- remapHits(target0, Lnodes.remapping=q_revperm,
+                                     new.nLnode=length(q_perm),
+                                     Rnodes.remapping=s_revperm,
+                                     new.nRnode=length(s_perm))
         current <- findOverlaps_NCList(query[q_perm], NCList(subject[s_perm]))
         checkTrue(.compare_hits(target, current))
         current <- findOverlaps_NCList(NCList(query[q_perm]), subject[s_perm])
@@ -155,63 +175,124 @@ test_findOverlaps_NCList <- function()
 test_findOverlaps_NCList_with_filtering <- function()
 {
     query <- IRanges(-3:7, width=3)
-    subject <- IRanges(rep.int(1:5, 5:1), c(1:5, 2:5, 3:5, 4:5, 5))
+    subject <- IRanges(rep.int(1:6, 6:1), c(0:5, 1:5, 2:5, 3:5, 4:5, 5))
 
     pp_query <- NCList(query)
     pp_subject <- NCList(subject)
-    for (min.score in -3:4) {
-        for (type in c("any", "start", "end", "within", "extend", "equal")) {
-            for (select in c("all", "first", "last", "count")) {
-                ## query - subject
-                target <- .findOverlaps_naive(query, subject,
-                                              min.score=min.score,
-                                              type=type, select=select)
-                current <- findOverlaps_NCList(query, pp_subject,
-                                               min.score=min.score,
-                                               type=type, select=select)
-                checkTrue(.compare_hits(target, current))
-                current <- findOverlaps_NCList(pp_query, subject,
-                                               min.score=min.score,
-                                               type=type, select=select)
-                checkTrue(.compare_hits(target, current))
-                current <- findOverlaps_NCList(query, subject,
-                                               min.score=min.score,
-                                               type=type, select=select)
-                checkTrue(.compare_hits(target, current))
-                ## subject - query
-                target <- .findOverlaps_naive(subject, query,
-                                              min.score=min.score,
-                                              type=type, select=select)
-                current <- findOverlaps_NCList(pp_subject, query,
-                                               min.score=min.score,
-                                               type=type, select=select)
-                checkTrue(.compare_hits(target, current))
-                current <- findOverlaps_NCList(subject, pp_query,
-                                               min.score=min.score,
-                                               type=type, select=select)
-                checkTrue(.compare_hits(target, current))
-                current <- findOverlaps_NCList(subject, query,
-                                               min.score=min.score,
-                                               type=type, select=select)
-                checkTrue(.compare_hits(target, current))
-                ## subject - subject
-                target <- .findOverlaps_naive(subject, subject,
-                                              min.score=min.score,
-                                              type=type, select=select)
-                current <- findOverlaps_NCList(pp_subject, subject,
-                                               min.score=min.score,
-                                               type=type, select=select)
-                checkTrue(.compare_hits(target, current))
-                current <- findOverlaps_NCList(subject, pp_subject,
-                                               min.score=min.score,
-                                               type=type, select=select)
-                checkTrue(.compare_hits(target, current))
-                current <- findOverlaps_NCList(subject, subject,
-                                               min.score=min.score,
-                                               type=type, select=select)
-                checkTrue(.compare_hits(target, current))
-            }
+    for (type in c("any", "start", "end", "within", "extend", "equal")) {
+      for (maxgap in 0:3) {
+        if (type != "any" || maxgap == 0L)
+            max_minoverlap <- 4L
+        else
+            max_minoverlap <- 1L
+        for (minoverlap in 0:max_minoverlap) {
+          for (select in c("all", "first", "last", "count")) {
+            ## query - subject
+            target <- .findOverlaps_naive(query, subject,
+                                          maxgap=maxgap, minoverlap=minoverlap,
+                                          type=type, select=select)
+            current <- findOverlaps_NCList(query, pp_subject,
+                                           maxgap=maxgap, minoverlap=minoverlap,
+                                           type=type, select=select)
+            checkTrue(.compare_hits(target, current))
+            current <- findOverlaps_NCList(pp_query, subject,
+                                           maxgap=maxgap, minoverlap=minoverlap,
+                                           type=type, select=select)
+            checkTrue(.compare_hits(target, current))
+            current <- findOverlaps_NCList(query, subject,
+                                           maxgap=maxgap, minoverlap=minoverlap,
+                                           type=type, select=select)
+            checkTrue(.compare_hits(target, current))
+            ## subject - query
+            target <- .findOverlaps_naive(subject, query,
+                                          maxgap=maxgap, minoverlap=minoverlap,
+                                          type=type, select=select)
+            current <- findOverlaps_NCList(pp_subject, query,
+                                           maxgap=maxgap, minoverlap=minoverlap,
+                                           type=type, select=select)
+            checkTrue(.compare_hits(target, current))
+            current <- findOverlaps_NCList(subject, pp_query,
+                                           maxgap=maxgap, minoverlap=minoverlap,
+                                           type=type, select=select)
+            checkTrue(.compare_hits(target, current))
+            current <- findOverlaps_NCList(subject, query,
+                                           maxgap=maxgap, minoverlap=minoverlap,
+                                           type=type, select=select)
+            checkTrue(.compare_hits(target, current))
+            ## subject - subject
+            target <- .findOverlaps_naive(subject, subject,
+                                          maxgap=maxgap, minoverlap=minoverlap,
+                                          type=type, select=select)
+            current <- findOverlaps_NCList(pp_subject, subject,
+                                           maxgap=maxgap, minoverlap=minoverlap,
+                                           type=type, select=select)
+            checkTrue(.compare_hits(target, current))
+            current <- findOverlaps_NCList(subject, pp_subject,
+                                           maxgap=maxgap, minoverlap=minoverlap,
+                                           type=type, select=select)
+            checkTrue(.compare_hits(target, current))
+            current <- findOverlaps_NCList(subject, subject,
+                                           maxgap=maxgap, minoverlap=minoverlap,
+                                           type=type, select=select)
+            checkTrue(.compare_hits(target, current))
+          }
         }
+      }
+    }
+}
+
+### Only test "start" and "end" types at the moment.
+test_findOverlaps_NCList_special_types <- function()
+{
+    x <- IRanges(10, 10)
+
+    x1 <- IRanges(10, 9)
+    y1 <- IRanges(start=c(7, 7, 13, 13), width=c(2, 0, 2, 0))
+    stopifnot(all(abs(start(x) - start(y1)) == 3L))
+    stopifnot(all(abs(start(x1) - start(y1)) == 3L))
+
+    x2 <- IRanges(11, 10)
+    y2 <- IRanges(end=c(7, 7, 13, 13), width=c(2, 0, 2, 0))
+    stopifnot(all(abs(end(x) - end(y2)) == 3L))
+    stopifnot(all(abs(end(x2) - end(y2)) == 3L))
+
+    test_maxgap_and_type <- function(maxgap, minoverlap, nhit)
+    {
+        hits <- findOverlaps(x, y1, maxgap=maxgap, minoverlap=minoverlap,
+                             type="start")
+        checkEquals(nhit, length(hits))
+        hits <- findOverlaps(y1, x, maxgap=maxgap, minoverlap=minoverlap,
+                             type="start")
+        checkEquals(nhit, length(hits))
+        hits <- findOverlaps(x1, y1, maxgap=maxgap, minoverlap=minoverlap,
+                             type="start")
+        checkEquals(nhit, length(hits))
+        hits <- findOverlaps(y1, x1, maxgap=maxgap, minoverlap=minoverlap,
+                             type="start")
+        checkEquals(nhit, length(hits))
+        hits <- findOverlaps(x, y2, maxgap=maxgap, minoverlap=minoverlap,
+                             type="end")
+        checkEquals(nhit, length(hits))
+        hits <- findOverlaps(y2, x, maxgap=maxgap, minoverlap=minoverlap,
+                             type="end")
+        checkEquals(nhit, length(hits))
+        hits <- findOverlaps(x2, y2, maxgap=maxgap, minoverlap=minoverlap,
+                             type="end")
+        checkEquals(nhit, length(hits))
+        hits <- findOverlaps(y2, x2, maxgap=maxgap, minoverlap=minoverlap,
+                             type="end")
+        checkEquals(nhit, length(hits))
+    }
+    ## no hits
+    for (maxgap in 0:2) {
+        test_maxgap_and_type(maxgap, minoverlap=1L, 0L)
+        test_maxgap_and_type(maxgap, minoverlap=0L, 0L)
+    }
+    for (maxgap in 3:5) {
+        ## no hits
+        test_maxgap_and_type(maxgap, minoverlap=1L, 0L)
+        ## 4 hits
+        test_maxgap_and_type(maxgap, minoverlap=0L, 4L)
     }
 }
 
@@ -219,32 +300,38 @@ test_findOverlaps_NCList_with_filtering <- function()
 {
     pp_query <- NCList(query)
     pp_subject <- NCList(subject)
-    for (min.score in -3:4) {
-        for (type in c("any", "start", "end", "within", "extend", "equal")) {
-            target <- as(.findOverlaps_naive(query, subject,
-                                             min.score=min.score,
-                                             type=type, select="all"),
-                         "CompressedIntegerList")
-            target_idx0 <- elementLengths(target) == 0L
-            check_arbitrary_hits <- function(current) {
-                current_idx0 <- is.na(current)
-                checkIdentical(target_idx0, current_idx0)
-                current <- as(current, "CompressedIntegerList")
-                checkTrue(all(current_idx0 | as.logical(current %in% target)))
-            }
-            current <- findOverlaps_NCList(query, pp_subject,
-                                           min.score=min.score,
-                                           type=type, select="arbitrary")
-            check_arbitrary_hits(current)
-            current <- findOverlaps_NCList(pp_query, subject,
-                                           min.score=min.score,
-                                           type=type, select="arbitrary")
-            check_arbitrary_hits(current)
-            current <- findOverlaps_NCList(query, subject,
-                                           min.score=min.score,
-                                           type=type, select="arbitrary")
-            check_arbitrary_hits(current)
+    for (type in c("any", "start", "end", "within", "extend", "equal")) {
+      for (maxgap in 0:3) {
+        if (type != "any" || maxgap == 0L)
+            max_minoverlap <- 4L
+        else
+            max_minoverlap <- 1L
+        for (minoverlap in 0:max_minoverlap) {
+          target <- as(.findOverlaps_naive(query, subject,
+                                           maxgap=maxgap, minoverlap=minoverlap,
+                                           type=type, select="all"),
+                       "CompressedIntegerList")
+          target_idx0 <- elementNROWS(target) == 0L
+          check_arbitrary_hits <- function(current) {
+              current_idx0 <- is.na(current)
+              checkIdentical(target_idx0, current_idx0)
+              current <- as(current, "CompressedIntegerList")
+              checkTrue(all(current_idx0 | as.logical(current %in% target)))
+          }
+          current <- findOverlaps_NCList(query, pp_subject,
+                                         maxgap=maxgap, minoverlap=minoverlap,
+                                         type=type, select="arbitrary")
+          check_arbitrary_hits(current)
+          current <- findOverlaps_NCList(pp_query, subject,
+                                         maxgap=maxgap, minoverlap=minoverlap,
+                                         type=type, select="arbitrary")
+          check_arbitrary_hits(current)
+          current <- findOverlaps_NCList(query, subject,
+                                         maxgap=maxgap, minoverlap=minoverlap,
+                                         type=type, select="arbitrary")
+          check_arbitrary_hits(current)
         }
+      }
     }
 }
 
@@ -254,7 +341,7 @@ test_findOverlaps_NCList_arbitrary <- function()
     subject <- IRanges(2:4, 10)
     .test_arbitrary_selection(query, subject)
     query <- IRanges(-3:7, width=3)
-    subject <- IRanges(rep.int(1:5, 5:1), c(1:5, 2:5, 3:5, 4:5, 5))
+    subject <- IRanges(rep.int(1:6, 6:1), c(0:5, 1:5, 2:5, 3:5, 4:5, 5))
     .test_arbitrary_selection(query, subject)
 }
 
@@ -333,7 +420,7 @@ test_NCLists <- function()
 {
     x1 <- IRanges(-3:7, width=3)
     x2 <- IRanges()
-    x3 <- IRanges(rep.int(1:5, 5:1), c(1:5, 2:5, 3:5, 4:5, 5))
+    x3 <- IRanges(rep.int(1:6, 6:1), c(0:5, 1:5, 2:5, 3:5, 4:5, 5))
     x <- IRangesList(x1=x1, x2=x2, x3=x3)
     mcols(x) <- DataFrame(label=c("first", "second", "third"))
     nclists <- NCLists(x)
@@ -348,7 +435,7 @@ test_NCLists <- function()
     checkIdentical(width(x), width(nclists))
     checkIdentical(x, as(nclists, "IRangesList"))
     checkIdentical(x[-1], as(nclists[-1], "IRangesList"))
-    checkIdentical(elementLengths(x), elementLengths(nclists))
+    checkIdentical(elementNROWS(x), elementNROWS(nclists))
 
     nclist <- nclists[[3]]
     checkTrue(is(nclist, "NCList"))
@@ -359,7 +446,7 @@ test_NCLists <- function()
 test_findOverlaps_NCLists <- function()
 {
     ir1 <- IRanges(-3:7, width=3)
-    ir2 <- IRanges(rep.int(1:5, 5:1), c(1:5, 2:5, 3:5, 4:5, 5))
+    ir2 <- IRanges(rep.int(1:6, 6:1), c(0:5, 1:5, 2:5, 3:5, 4:5, 5))
     target0 <- mapply(findOverlaps_NCList, list(ir1, ir2), list(ir2, ir1))
     for (compress in c(TRUE, FALSE)) {
         query <- IRangesList(ir1, ir2, IRanges(2, 7), compress=compress)
