@@ -1,8 +1,6 @@
 #include "../inst/include/IRanges_defines.h"
 #include <string.h>
 
-#define DEBUG_IRANGES 1
-
 #define INIT_STATIC_SYMBOL(NAME) \
 { \
 	if (NAME ## _symbol == NULL) \
@@ -32,7 +30,7 @@ int _invert_overlap_code(
 	int code
 );
 
-SEXP Ranges_compare(
+SEXP Ranges_pcompare(
 	SEXP x_start,
 	SEXP x_width,
 	SEXP y_start,
@@ -42,8 +40,6 @@ SEXP Ranges_compare(
 
 /* IRanges_class.c */
 
-SEXP debug_IRanges_class();
-
 SEXP _get_IRanges_start(SEXP x);
 
 SEXP _get_IRanges_width(SEXP x);
@@ -143,8 +139,6 @@ SEXP solve_user_SEW(
 
 /* Grouping_class.c */
 
-SEXP debug_Grouping_class();
-
 SEXP _get_H2LGrouping_high2low(SEXP x);
 
 SEXP _get_H2LGrouping_low2high(SEXP x);
@@ -172,7 +166,7 @@ SEXP H2LGrouping_vmembers(
 
 /* SimpleIRangesList_class.c */
 
-SEXP SimpleIRangesList_isNormal(SEXP x);
+SEXP SimpleIRangesList_isNormal(SEXP x, SEXP use_names);
 
 SEXP SimpleNormalIRangesList_min(SEXP x);
 
@@ -255,7 +249,7 @@ IRanges_holder _get_elt_from_CompressedIRangesList_holder(
 	int i
 );
 
-int _get_eltlens_from_CompressedIRangesList_holder(
+int _get_eltNROWS_from_CompressedIRangesList_holder(
 	const CompressedIRangesList_holder *x_holder,
 	int i
 );
@@ -285,19 +279,8 @@ SEXP CompressedNormalIRangesList_max(
 SEXP valid_GappedRanges(SEXP x, SEXP ans_type);
 
 
-/* RangedData_class.c */
-
-SEXP _new_RangedData(
-	const char *classname,
-	SEXP ranges,
-	SEXP values
-);
-
-
 /* inter_range_methods.c */
 
-SEXP debug_inter_range_methods();
-
 SEXP IRanges_range(SEXP x);
 
 SEXP Ranges_reduce(
@@ -369,11 +352,11 @@ SEXP NCList_build(
 	SEXP x_subset
 );
 
-SEXP new_NCListSXP_from_NCList(
+SEXP new_NCListAsINTSXP_from_NCList(
 	SEXP nclist_xp
 );
 
-SEXP NCListSXP_print(
+SEXP NCListAsINTSXP_print(
 	SEXP x_nclist,
 	SEXP x_start,
 	SEXP x_end
@@ -386,7 +369,8 @@ SEXP NCList_find_overlaps(
 	SEXP s_end,
 	SEXP nclist,
 	SEXP nclist_is_q,
-	SEXP min_score,
+	SEXP maxgap,
+	SEXP minoverlap,
 	SEXP type,
 	SEXP select,
 	SEXP circle_length
@@ -403,7 +387,8 @@ SEXP NCList_find_overlaps_in_groups(
 	SEXP s_groups,
 	SEXP nclists,
 	SEXP nclist_is_q,
-	SEXP min_score,
+	SEXP maxgap,
+	SEXP minoverlap,
 	SEXP type,
 	SEXP select,
 	SEXP circle_length
@@ -423,26 +408,72 @@ SEXP CompressedNumericList_sum(
   SEXP x,
   SEXP na_rm);
 
+SEXP CompressedLogicalList_prod(
+  SEXP x,
+  SEXP na_rm);
+
+SEXP CompressedIntegerList_prod(
+  SEXP x,
+  SEXP na_rm);
+
+SEXP CompressedNumericList_prod(
+  SEXP x,
+  SEXP na_rm);
+
 SEXP CompressedLogicalList_min(
   SEXP x,
   SEXP na_rm);
 
+SEXP CompressedLogicalList_which_min(
+  SEXP x);
+
 SEXP CompressedIntegerList_min(
   SEXP x,
   SEXP na_rm);
 
+SEXP CompressedIntegerList_which_min(
+  SEXP x);
+
 SEXP CompressedNumericList_min(
   SEXP x,
   SEXP na_rm);
 
+SEXP CompressedNumericList_which_min(
+  SEXP x);
+
 SEXP CompressedLogicalList_max(
   SEXP x,
   SEXP na_rm);
 
+SEXP CompressedLogicalList_which_max(
+  SEXP x);
+
 SEXP CompressedIntegerList_max(
   SEXP x,
   SEXP na_rm);
 
+SEXP CompressedIntegerList_which_max(
+  SEXP x);
+
 SEXP CompressedNumericList_max(
   SEXP x,
   SEXP na_rm);
+
+SEXP CompressedNumericList_which_max(
+  SEXP x);
+
+SEXP CompressedLogicalList_is_unsorted(
+  SEXP x,
+  SEXP na_rm,
+  SEXP strictly);
+
+SEXP CompressedIntegerList_is_unsorted(
+  SEXP x,
+  SEXP na_rm,
+  SEXP strictly);
+
+SEXP CompressedNumericList_is_unsorted(
+  SEXP x,
+  SEXP na_rm,
+  SEXP strictly);
+
