@@ -11,10 +11,9 @@ static const R_CallMethodDef callMethods[] = {
 	CALLMETHOD_DEF(valid_Ranges, 3),
 
 /* Ranges_comparison.c */
-	CALLMETHOD_DEF(Ranges_compare, 4),
+	CALLMETHOD_DEF(Ranges_pcompare, 4),
 
 /* IRanges_class.c */
-	CALLMETHOD_DEF(debug_IRanges_class, 0),
 	CALLMETHOD_DEF(IRanges_isNormal, 1),
 	CALLMETHOD_DEF(IRanges_from_integer, 1),
 	CALLMETHOD_DEF(NormalIRanges_from_logical, 1),
@@ -24,8 +23,6 @@ static const R_CallMethodDef callMethods[] = {
 	CALLMETHOD_DEF(solve_user_SEW, 6),
 
 /* Grouping_class.c */
-	CALLMETHOD_DEF(debug_Grouping_class, 0),
-
 	CALLMETHOD_DEF(H2LGrouping_members, 2),
 	CALLMETHOD_DEF(H2LGrouping_vmembers, 2),
 
@@ -38,7 +35,7 @@ static const R_CallMethodDef callMethods[] = {
 	CALLMETHOD_DEF(RleViews_viewWhichMaxs, 2),
 
 /* SimpleIRangesList_class.c */
-	CALLMETHOD_DEF(SimpleIRangesList_isNormal, 1),
+	CALLMETHOD_DEF(SimpleIRangesList_isNormal, 2),
 	CALLMETHOD_DEF(SimpleNormalIRangesList_min, 1),
 	CALLMETHOD_DEF(SimpleNormalIRangesList_max, 1),
 
@@ -52,7 +49,6 @@ static const R_CallMethodDef callMethods[] = {
 	CALLMETHOD_DEF(valid_GappedRanges, 2),
 
 /* inter_range_methods.c */
-	CALLMETHOD_DEF(debug_inter_range_methods, 0),
 	CALLMETHOD_DEF(IRanges_range, 1),
 	CALLMETHOD_DEF(Ranges_reduce, 6),
 	CALLMETHOD_DEF(CompressedIRangesList_reduce, 4),
@@ -68,21 +64,33 @@ static const R_CallMethodDef callMethods[] = {
 	CALLMETHOD_DEF(NCList_new, 0),
 	CALLMETHOD_DEF(NCList_free, 1),
 	CALLMETHOD_DEF(NCList_build, 4),
-	CALLMETHOD_DEF(new_NCListSXP_from_NCList, 1),
-	CALLMETHOD_DEF(NCListSXP_print, 3),
-	CALLMETHOD_DEF(NCList_find_overlaps, 10),
-	CALLMETHOD_DEF(NCList_find_overlaps_in_groups, 14),
+	CALLMETHOD_DEF(new_NCListAsINTSXP_from_NCList, 1),
+	CALLMETHOD_DEF(NCListAsINTSXP_print, 3),
+	CALLMETHOD_DEF(NCList_find_overlaps, 11),
+	CALLMETHOD_DEF(NCList_find_overlaps_in_groups, 15),
 
 /* CompressedAtomicList_utils.c */
 	CALLMETHOD_DEF(CompressedLogicalList_sum, 2),
-  CALLMETHOD_DEF(CompressedIntegerList_sum, 2),
-  CALLMETHOD_DEF(CompressedNumericList_sum, 2),
+	CALLMETHOD_DEF(CompressedIntegerList_sum, 2),
+	CALLMETHOD_DEF(CompressedNumericList_sum, 2),
+	CALLMETHOD_DEF(CompressedLogicalList_prod, 2),
+	CALLMETHOD_DEF(CompressedIntegerList_prod, 2),
+	CALLMETHOD_DEF(CompressedNumericList_prod, 2),
 	CALLMETHOD_DEF(CompressedLogicalList_min, 2),
-  CALLMETHOD_DEF(CompressedIntegerList_min, 2),
-  CALLMETHOD_DEF(CompressedNumericList_min, 2),
+	CALLMETHOD_DEF(CompressedLogicalList_which_min, 1),
+	CALLMETHOD_DEF(CompressedIntegerList_min, 2),
+	CALLMETHOD_DEF(CompressedIntegerList_which_min, 1),
+	CALLMETHOD_DEF(CompressedNumericList_min, 2),
+	CALLMETHOD_DEF(CompressedNumericList_which_min, 1),
 	CALLMETHOD_DEF(CompressedLogicalList_max, 2),
-  CALLMETHOD_DEF(CompressedIntegerList_max, 2),
-  CALLMETHOD_DEF(CompressedNumericList_max, 2),
+	CALLMETHOD_DEF(CompressedLogicalList_which_max, 1),
+	CALLMETHOD_DEF(CompressedIntegerList_max, 2),
+	CALLMETHOD_DEF(CompressedIntegerList_which_max, 1),
+	CALLMETHOD_DEF(CompressedNumericList_max, 2),
+	CALLMETHOD_DEF(CompressedNumericList_which_max, 1),
+	CALLMETHOD_DEF(CompressedLogicalList_is_unsorted, 3),
+	CALLMETHOD_DEF(CompressedIntegerList_is_unsorted, 3),
+	CALLMETHOD_DEF(CompressedNumericList_is_unsorted, 3),
 
 	{NULL, NULL, 0}
 };
@@ -134,10 +142,9 @@ void R_init_IRanges(DllInfo *info)
 
 /* CompressedIRangesList_class.c */
 	REGISTER_CCALLABLE(_hold_CompressedIRangesList);
+	REGISTER_CCALLABLE(_get_length_from_CompressedIRangesList_holder);
 	REGISTER_CCALLABLE(_get_elt_from_CompressedIRangesList_holder);
-
-/* RangedData_class.c */
-	REGISTER_CCALLABLE(_new_RangedData);
+	REGISTER_CCALLABLE(_get_eltNROWS_from_CompressedIRangesList_holder);
 
 	return;
 }
